###############################################################################
# CG150.1 Headaches: Contrast Level Threshold Analyses
###############################################################################

# Install the nmathresh package from CRAN if not already installed
# install.packages("nmathresh")

library(nmathresh)


## SETUP ----------------------------------------------------------------------
# Read in the posterior summaries. These were generated by WinBUGS, and saved
# in a summary format for easier re-use.
load("./Headaches posterior summaries.Rda")

# Posterior means of treatment parameters
post.mean.d <- post.summary$statistics[1:7, "Mean"]
post.cov.d <- post.cov[1:7, 1:7]


## RECONSTRUCT LIKELIHOOD  ----------------------------------------------------

# First, we must reconstruct the likelihood covariance matrix. Note that a flat
# prior with precision 0.0001 was used.

# Contrast design matrix is
X <- matrix(ncol = 7,byrow = TRUE,
            # d2 d3 d4 d5 d6 d7 d8
            c(1, 0, 0, 0, 0, 0, 0,
              0, 0, 1, 0, 0, 0, 0,
              0, 0, 0, 1, 0, 0, 0,
              0, 0, 0, 0, 1, 0, 0,
              0, 0, 0, 0, 0, 1, 0,
              0, 0, 0, 0, 0, 0, 1,
              0,-1, 0, 0, 1, 0, 0,
              0, 0, 0, 0,-1, 1, 0))

# Reconstruct using NNLS
lik.cov <- recon_vcov(post.cov.d,          # Posterior covariance matrix
                      prior.prec = .0001,  # Prior precision
                      X = X)               # Contrast design matrix

# Get indices of contrasts in likelihood in full contrast vector
d.a <- d.b <- vector(length = nrow(X))
for (i in 1:nrow(X)) {
  d.a[i] <- ifelse(any(X[i,] == -1), which(X[i,] == -1),0) + 1
  d.b[i] <- ifelse(any(X[i,] == 1),which(X[i,] == 1),0) + 1
}

d.i <- d_ab2i(d.a, d.b, K = 8)


## THRESHOLD ANALYSIS ---------------------------------------------------------

# Now we can perform threshold analysis at the contrast level.

# Thresholds are derived using the nma_thresh function
thresh <- nma_thresh(mean.dk = post.mean.d,   # Posterior means of treatment effects
                     lhood = lik.cov,         # Reconstructed likelihood covariance matrix
                     post = post.cov.d,       # Posterior covariance matrix
                                              # -- Further options below --
                     nmatype = "fixed",       # Approximate two-stage FE NMA
                     X = X,                   # Contrast design matrix
                     opt.max = FALSE,         # Lower treatment effects better
                     mcid = 0.5,              # Specify MCID for decision rule
                     mcid.type = "decision")


## PLOT RESULTS ---------------------------------------------------------------

# Label the contrasts and display using a forest plot, along with 95% posterior
# credible intervals as reported in CG150.1
plotdat <- data.frame(
            contr = paste0(d.b," vs. ",d.a),
            contr.mean = post.summarydd$statistics[d.i, "Mean"],
            CI2.5 = post.summarydd$quantiles[d.i, "2.5%"],
            CI97.5 = post.summarydd$quantiles[d.i, "97.5%"])

# GRADE quality
GRADEquality <- c("Low", # 2 v 1
                  "Moderate", # 4 v 1
                  "Very low", # 5 v 1
                  "Low", # 6 v 1
                  "Very low", # 7 v 1
                  "Low", # 8 v 1
                  "Moderate", # 6 v 3
                  "Moderate" # 7 v 6
                  )

# Plot
pdf("Headaches thresholds - contrast level - efficacy.pdf", 
    width = 14, height = 3.8)

fp <- thresh_forest(
          thresh = thresh,     # Threshold object produced by nma_thresh
          y = contr.mean,      # Column of data points
          CI.lo = CI2.5, CI.hi = CI97.5,  # Columns of lower and upper limits of CIs
          label = contr,       # Column of labels
          data = plotdat,      # Data frame containing above data
                               # -- Plotting options below here --
          label.title = "Contrast", 
          xlab = "Change in headache days per month",
          CI.title = "95% Credible Interval",
          xlim = c(-3, 2), 
          refline = 0, 
          digits = 2, cutoff = 30,
          II.cols = rgb(0.72, 0.8, 0.93),
          add.columns = GRADEquality, add.columns.title = "GRADE", 
          add.columns.hjust = 0,
          display = FALSE)

# Add optimal treatment caption below table
nrow_fp <- nrow(fp)
fp <- gtable::gtable_add_rows(fp, grid::unit(1, "null"))
fp <- gtable::gtable_add_grob(fp, 
                              grid::textGrob(paste0("Base-case optimal treatment set is ", 
                                                    paste(thresh$kstar, collapse = ", "), "."), 
                                             x = 0, just = "left"),
                              t = nrow_fp + 1, l = 4, clip = "off")
fp <- gtable::gtable_add_rows(fp, grid::unit(1, "null"))

# Display
grid::grid.newpage()
grid::grid.draw(fp)

dev.off()
