###############################################################################
# CG150.1 Headache: Study Level Threshold Analyses
###############################################################################

# Install the nmathresh package from CRAN if not already installed
# install.packages("nmathresh")

library(nmathresh)


## SETUP ----------------------------------------------------------------------
# Here we perform initial setup, reading in the data and posterior summaries
#  - Read in data, arm and trial level separately.
#  - Transpose rows to columns for arm level data (not necessary for trial 
#    level, we only have 2-arm trials)
#  - Concatenate into one dataset
#  - Read in posterior covariance matrix from WinBUGS

# Arm level
dat.arm <- read.table("./Headaches data ARM LEVEL.txt", 
                      header = TRUE, stringsAsFactors = FALSE)

dat.arm$studyID <- 1:nrow(dat.arm) # study ID
dat.arm$b <- dat.arm$t.1 # baseline treatment

# Trial level
dat.trial <- read.table("./Headaches data TRIAL LEVEL.txt", 
                        header = TRUE, stringsAsFactors = FALSE)

dat.trial$studyID <- nrow(dat.arm) + 1:nrow(dat.trial) # study ID
dat.trial <- with(dat.trial, 
                  data.frame(studyID, textID, b = t.1, t = t.2, y = y.2, se = se.2, 
                             n_arms, armlvl = FALSE, arm = NA, bl = FALSE)
                  )


# Transpose wide into long data (one row per arm)
dat.arml <- reshape(dat.arm, varying = c("t.1","y.1","se.1","t.2","y.2","se.2",
                                       "t.3","y.3","se.3","t.4","y.4","se.4"), 
                    timevar = "arm", idvar = "studyID", direction = "long")

# Sort data by study and contrast, removing NA rows
dat.arml <- dat.arml[order(dat.arml$studyID, dat.arml$arm, dat.arml$y, na.last = NA),]

# Flag for baseline rows
dat.arml$bl <- rep(FALSE,nrow(dat.arml))
dat.arml$bl[1] <- TRUE
dat.arml$bl[cumsum(table(dat.arml$studyID))[-nrow(dat.arm)] + 1] <- TRUE

# Flag for arm level data rows
dat.arml$armlvl <- TRUE

# Concatenate into one dataset
# NOTE: Arm level rows come first then trial level, just like in the
# WinBUGS code, so that e.g. delta parameters are in correct order.
dat <- rbind(dat.arml, dat.trial)

# Read in risk of bias table
dat.rob <- read.csv("./study_rob_table.csv", 
                    header = TRUE, stringsAsFactors = FALSE)

dat <- merge(dat, dat.rob, by.x = "textID", by.y = "study", all.x = TRUE, sort = FALSE)

# Only show RoB for first row of each study
for (i in 2:nrow(dat)) {
  if (dat[i, "textID"] == dat[i - 1, "textID"]) {
    dat[i, c("sel", "perf", "det", "att", "rep", "oth")] <- " "
  }
}


# Read in the posterior summaries. These were generated by WinBUGS, and saved
# in a summary format for easier re-use.
load("./Headaches posterior summaries.Rda")

# Posterior means of treatment parameters
post.mean.d <- post.summary$statistics[1:7, "Mean"]


## THRESHOLD ANALYSIS ---------------------------------------------------------

# Define constants
N <- nrow(dat)                        # number of data points
n1 <- nrow(dat.arm)                   # number of arm level studies
n2 <- nrow(dat.trial)                 # number of trial level studies
n <- n1 + n2                          # total number of studies
K <- length(unique(c(dat$t, dat$b)))  # number of treatments

# Likelihood covariance matrix V. This is simply diagonal as arms are
# independent in arm level data and we only have 2-arm trials for trial level
# data.
V <- diag(dat$se^2)

# Baseline parameter mu design matrix M. Remember that only the arm level data
# points have a mu!
M <- matrix(0, nrow = N, ncol = n1)
for (i in 1:N) {
  if (dat$armlvl[i]) M[i, dat$studyID[i]] <- 1
}

# Random effect delta parameter design matrix L. Baseline rows from arm level
# data don't have a delta, all rows from trial level data have a delta.
L <- matrix(0, nrow = N, ncol = N - n1)
j <- 1
for (i in 1:N) {
  if (!dat$bl[i]) {
    L[i, j] <- 1
    j <- j + 1
  }
}


## Perform threshold analysis
thresh <- nma_thresh(mean.dk = post.mean.d,   # Posterior means of treatment effects
                     lhood = V,               # Likelihood covariance matrix
                     post = post.cov,         # Posterior covariance matrix
                                              # -- Further options below --
                     nmatype = "random",      # Random effects NMA
                     mu.design = M,           # Design matrix of baseline parameters
                     delta.design = L,        # Design matrix for random effects
                     opt.max = FALSE,         # Lower treatment effects better
                     mcid = 0.5,              # Specify MCID for decision rule
                     mcid.type = "decision")


## PLOT RESULTS ---------------------------------------------------------------

# Make nice row labels
dat$lab <- ""
for (i in 1:N) {
  if (dat$armlvl[i]) dat$lab[i] <- paste0(dat$textID[i]," (",dat$t[i],")")
  else dat$lab[i] <- paste0(dat$textID[i],"\n(",dat$t[i]," vs. ",dat$b[i],")")
}

dat$lab2 <- ""
dat$textID.SHORT <- sub("^(....).*(..)$", "\\1\\2", dat$textID)
for (i in 1:N) {
  if (dat$armlvl[i]) {
    dat$lab2[i] <- paste0(dat$textID.SHORT[i]," (",dat$t[i],")")
  }
  else {
    dat$lab2[i] <- paste0(dat$textID.SHORT[i]," (",dat$t[i]," vs. ",dat$b[i],")")
  }
}

# Confidence intervals about study estimates
dat$CI.lo <- dat$y + qnorm(0.025) * dat$se
dat$CI.hi <- dat$y + qnorm(0.975) * dat$se

# Plot
pdf("Headaches thresholds - study level - efficacy.pdf", 
    width = 16, height = 10.3)

fp <- thresh_forest(
          thresh = thresh,     # Threshold object produced by nma_thresh
          y = y,               # Column of data points
          CI.lo = CI.lo, CI.hi = CI.hi,   # Columns of lower and upper limits of CIs
          label = lab,         # Column of labels
          data = dat,          # Data frame containing above data
                               # -- Plotting options below here --
          label.title = "Study (Trt.)", 
          CI.title = "95% Confidence Interval",
          xlab = "Change in headache days per month", 
          xlim = c(-8, 2), 
          refline = 0, 
          digits = 2, cutoff = 30, xbreaks = -8:2,
          II.cols = rgb(0.72, 0.8, 0.93),
          add.columns = dat[,c("sel", "perf", "det", "att", "rep", "oth")],
          add.columns.title = c("Sel", "Perf", "Det", "Att", "Rep", "Oth"),
          add.columns.hjust = 0,
          display = FALSE
          )

# Add optimal treatment caption below table
nrow_fp <- nrow(fp)
fp <- gtable::gtable_add_rows(fp, grid::unit(1, "null"))
fp <- gtable::gtable_add_grob(fp, 
                              grid::textGrob(paste0("Base-case optimal treatment set is ", 
                                                    paste(thresh$kstar, collapse = ", "), "."), 
                                             x = 0, just = "left"),
                              t = nrow_fp + 1, l = 4, clip = "off")
fp <- gtable::gtable_add_rows(fp, grid::unit(1, "null"))

# Display
grid::grid.newpage()
grid::grid.draw(fp)

dev.off()
